USE tempdb;

CREATE TABLE dbo.Students(
StudentID int NOT NULL,
FirstName varchar(10) NOT NULL,
LastName varchar(10) NOT NULL,
NickName varchar(20) NULL,
CONSTRAINT PK_Students PRIMARY KEY NONCLUSTERED(StudentID)
);

INSERT dbo.Students VALUES
      (1, 'Lorenzo', 'Benaglia', 'Lord Blizzard')
    , (2, 'David', 'De Giacomi', 'Brainkiller')
    , (3, 'Alessandro', 'Alpi', 'Sux Stellino')
    , (4, 'Matteo', 'Raumer', NULL);

ALTER TABLE dbo.Students
ADD CONSTRAINT UNQ_NickName UNIQUE(NickName);

INSERT dbo.Students VALUES
      (5, 'Marco', 'Caruso', NULL);
      
/* Elimino il constraint UNIQUE */
ALTER TABLE dbo.Students
DROP CONSTRAINT UNQ_NickName;
GO

/* Definisco un after trigger di INSERT/UPDATE */
CREATE TRIGGER dbo.trIU_Students
ON dbo.Students
AFTER INSERT, UPDATE
AS
IF UPDATE(NickName) BEGIN
    IF EXISTS(
        SELECT *
        FROM dbo.Students AS S
        JOIN INSERTED AS I
        ON S.NickName = I.NickName
        WHERE S.NickName IS NOT NULL) BEGIN
        
        RAISERROR ('Non sono ammessi soprannomi duplicati.', 16, 1);
        ROLLBACK TRAN;
    END
END
GO

/* Inserisco una riga con NickName a NULL */
INSERT dbo.Students VALUES
      (5, 'Marco', 'Caruso', NULL);

/* Inserisco una riga con NickName duplicato */
INSERT dbo.Students VALUES
      (6, 'Giovanni', 'Ferron', 'Brainkiller');

/* Elimino il trigger */
DROP TRIGGER dbo.trIU_Students;
GO

/* Creo la vista */
CREATE VIEW dbo.vwNickName
WITH SCHEMABINDING
AS
SELECT NickName
FROM dbo.Students
WHERE NickName IS NOT NULL;
GO

/* La materializzo */
CREATE UNIQUE CLUSTERED INDEX ICL_NickName
ON dbo.vwNickName(NickName);

/* Inserisco una riga con NickName a NULL */
INSERT dbo.Students VALUES
      (6, 'Ester', 'Memoli', NULL);
      
/* Inserisco una riga con NickName duplicato */
INSERT dbo.Students VALUES
      (7, 'Giovanni', 'Ferron', 'Brainkiller');

/* Elimino la vista */
DROP VIEW dbo.vwNickName;

/* Creo l'indice */
CREATE UNIQUE NONCLUSTERED INDEX idx_NickName_notnull
ON dbo.Students(NickName)
WHERE NickName IS NOT NULL;

/* Inserisco una riga con NickName a NULL */
INSERT dbo.Students VALUES
      (7, 'Stefano', 'Passatordi', NULL);
      
/* Inserisco una riga con NickName duplicato */
INSERT dbo.Students VALUES
      (8, 'Giovanni', 'Ferron', 'Brainkiller');
      
/* Pulizia */
DROP TABLE dbo.Students;